unit WeatherUtils;

interface

uses SysUtils, Classes, {Dialogs,} IniFiles, OverbyteIcsUrl, OverbyteIcsHttpProt,
  OverbyteIcsWSocket;

function GetStringBetween(const GetFrom, aStartString, aEndString: string): string;

type
  TProxyType = (PT_NONE, PT_HTTP, PT_SOCKS4, PT_SOCKS5);

  TProxySettings = record
    ProxyType: TProxyType;
    Address: string;
    Port: string;
    {----------------}
    NeedAuth: boolean;
    UserName: string;
    UserPwd: string;
  end;

  TWeather = class(TObject)
  private
    FParserIni: TMemIniFile;
    FHttpCli: THttpCli;
    FWeather: TStringList;
    FWeatherParams: TStringList;
    FTownList: TStringList;
    FProxySettings: TProxySettings;

    procedure GetWeatherParams;
  public
    constructor Create(aParserIniFileName: string);
    destructor Destroy; override;

    procedure GetWeatherForTownList(aRetryCount: integer = 0;
      aRetryInterval: integer = 0);
    procedure GetWeather(const aTownId: string; aRetryCount: integer = 0;
      aRetryInterval: integer = 0);
    procedure ParseWeather(const aText: string);

    property Weather: TStringList read FWeather;
    property TownList: TStringList read FTownList;
    property ProxySettings: TProxySettings read FProxySettings write FProxySettings;
  end;

const
  CurrentWeatherMarker: string = '_CW ';

implementation

uses StrUtils;

function fAnsiPos(const Substr, S: string; FromPos: integer): Integer;
var
  P: PChar;
begin
  Result := 0;
  P := AnsiStrPos(PChar(S) + fromPos - 1, PChar(SubStr));
  if P <> nil then
    Result := Integer(P) - Integer(PChar(S)) + 1;
end;
//-----------------------------------------------------------------------------
function GetStringBetween(const GetFrom, aStartString, aEndString: string): string;
var
  IndStart: integer;
  IndEnd: integer;
begin
  IndStart := AnsiPos(aStartString, GetFrom) + Length(aStartString);
  IndEnd   := PosEx{fAnsiPos}(aEndString, GetFrom, IndStart);
  //if IndEnd = 0 then ShowMessage(aEndString);
  Result   := Copy(GetFrom, IndStart, IndEnd - IndStart);
end;

{-----------------------------------------------------------------------------
 TWeather
 -----------------------------------------------------------------------------}

constructor TWeather.Create(aParserIniFileName: string);
begin
  if FileExists(aParserIniFileName) then
    FParserIni := TMemIniFile.Create(aParserIniFileName)
  else raise Exception.Create('     ');

  if FParserIni.ReadInteger('Weather Parser', 'FormatVersion', 0) < 1 then
    raise Exception.Create('     ');

  FWeather       := TStringList.Create;
  FWeatherParams := TStringList.Create;
  FTownList      := TStringList.Create;

  GetWeatherParams;

  FHttpCli := THttpCli.Create(nil);

  FProxySettings.ProxyType := PT_NONE;
  FProxySettings.NeedAuth  := false;
end;
//-----------------------------------------------------------------------------
procedure TWeather.GetWeatherParams;
var
  i: integer;
begin
  //   
  FWeatherParams.Clear;
  FParserIni.ReadSections(FWeatherParams);
  i := 0;
  while i < FWeatherParams.Count do
  begin
    if not AnsiStartsText(CurrentWeatherMarker, FWeatherParams[i]) then
      FWeatherParams.Delete(i)
    else Inc(i);
  end;

end;
//-----------------------------------------------------------------------------
destructor TWeather.Destroy;
begin
  if FParserIni <> nil then
  begin
    FParserIni.Free;
    FParserIni := nil;
  end;

  if FHttpCli <> nil then
  begin
    FHttpCli.Free;
    FHttpCli := nil;
  end;

  if FWeather <> nil then
  begin
    FWeather.Free;
    FWeather := nil;
  end;

  if FWeatherParams <> nil then
  begin
    FWeather.Free;
    FWeather := nil;
  end;

  if FTownList <> nil then
  begin
    FTownList.Free;
    FTownList := nil;
  end;

  inherited Destroy;
end;
//-----------------------------------------------------------------------------
procedure TWeather.GetWeather(const aTownId: string; aRetryCount: integer = 0;
  aRetryInterval: integer = 0);
var
  Weather: TStringList;
  TryCount: integer;
begin
  try
    Weather               := TStringList.Create;
    FHttpCli.RcvdStream   := TMemoryStream.Create;
  except
    raise Exception.Create('   ');
  end;

  //  -
  case FProxySettings.ProxyType of
    PT_HTTP:
      begin
        FHttpCli.Proxy       := ProxySettings.Address;
        FHttpCli.ProxyPort   := ProxySettings.Port;

        if ProxySettings.NeedAuth then
        begin
          FHttpCli.ProxyAuth     := httpAuthBasic;
          FHttpCli.ProxyUsername := FProxySettings.UserName;
          FHttpCli.ProxyPassword := FProxySettings.UserPwd;
        end;
      end;
    PT_SOCKS4, PT_SOCKS5:
      begin
        FHttpCli.SocksServer := ProxySettings.Address;
        FHttpCli.SocksPort   := ProxySettings.Port;

        if ProxySettings.NeedAuth then
        begin
          FHttpCli.SocksAuthentication := socksAuthenticateUsercode;
          FHttpCli.SocksUsercode       := FProxySettings.UserName;
          FHttpCli.SocksPassword       := FProxySettings.UserPwd;
        end;

        if FProxySettings.ProxyType = PT_SOCKS4 then FHttpCli.SocksLevel := '4'
        else FHttpCli.SocksLevel := '5';
      end;
  end;

  // URL    
  FHttpCli.URL            := Format(FParserIni.ReadString('Default', 'Default URL', ''),
    [aTownId]);
  //FHttpCli.AcceptLanguage := 'en, ru';
  FHttpCli.Connection     := 'Keep-Alive';
  FHttpCli.RequestVer     := '1.1';

  //  
  try
    TryCount := 0;
    repeat begin
      FHttpCli.Get;
      
      Inc(TryCount);
      if TryCount >= aRetryCount then Break;
      if FHttpCli.RcvdStream.Size = 0 then Sleep(1000*aRetryInterval);
    end until FHttpCli.RcvdStream.Size <> 0;
  except
    Exception.Create('     , StatusCode = ' +
      IntToStr(FHttpCli.StatusCode) + ' ReasonPhrase = ' + FHttpCli.ReasonPhrase);
    Exit;
  end;

  //     -   
  try
    FHttpCli.RcvdStream.Position := 0;

    if FHttpCli.RcvdStream.Size <> 0 then
    begin
      Weather.LoadFromStream(FHttpCli.RcvdStream);
      ParseWeather(Weather.Text);
    end else FWeather.Text := '     ';
    //Weather.SaveToFile('3.htm');
    //ShowMessage(Weather.Text);
  except
    raise Exception.Create('     ');
  end;

  // 
  if FHttpCli.RcvdStream <> nil then
  begin
    FHttpCli.RcvdStream.Free;
    FHttpCli.RcvdStream := nil;
  end;

  if Weather <> nil then
  begin
    Weather.Free;
    Weather := nil;
  end;
end;
//-----------------------------------------------------------------------------
procedure TWeather.ParseWeather(const aText: string);
  function ParseWeatherParameter(const aParamName: string; aUnit: string = ''): string;
  var
    OldUnit: string;
  begin
    with FParserIni do
    begin
      Result := GetStringBetween(aText,
        ReadString(aParamName, 'Start', ''), ReadString(aParamName, 'End', ''));
      if Length(Result) = 0 then
      begin
        Result := ReadString('Default', 'NoData', 'No');
        Exit;
      end;

      OldUnit := ReadString(aParamName, 'OldUnit', '');
      if Length(OldUnit) > 0 then
        Result := AnsiReplaceText(Result, OldUnit, ReadString(aParamName, 'Unit', aUnit))
      else Result := Result + ReadString(aParamName, 'Unit', aUnit);
    end;
  end;
  //---------------------------------------------------------------------------
  function GetParamTitle(const aParamName: string): string;
  begin
    with FParserIni do
    begin
      Result := ReadString(aParamName, 'Title', aParamName);
      if Length(Result) <> 0 then
        Result := Result + ReadString('Default', 'Separator', ':') + ' ';
    end;
  end;
var
  i: integer;
  TmpStr: string;
begin
  for i := 0 to FWeatherParams.Count - 1 do
  begin
    TmpStr := FWeatherParams[i];
    FWeather.Add(GetParamTitle(TmpStr) + ParseWeatherParameter(TmpStr));
  end;
end;
//-----------------------------------------------------------------------------
procedure TWeather.GetWeatherForTownList(aRetryCount: integer = 0;
  aRetryInterval: integer = 0);
var
  i: integer;
begin
  FWeather.Clear;
  for i := 0 to FTownList.Count - 1 do
  begin
    //FWeather.Add('[' + FTownList[i] + ']');
    GetWeather(FTownList[i], aRetryCount, aRetryInterval);
    //FWeather.Add('[/' + FTownList[i] + ']');
  end;
end;

end.
